using System;
using System.Data;
using System.Data.OleDb;
using System.Text;
using System.Collections;
using System.Configuration;
using System.ComponentModel;
using System.Globalization;
using Cryptographys;
using System.Diagnostics;
namespace TimecheckDataProcess
{
    class DataFunction
    {
        OleDbConnection connClass;
        string strDecryptedString = string.Empty;
        public DataFunction()
        {
            strDecryptedString = Cryptographys.Cryptography.GetDecryptedConnectionString();
        }
        //Code for OleDbConnection Open
        public void OpenCon(OleDbConnection ConnOpen)
        {
            if (ConnOpen.State == ConnectionState.Closed)
            {
                ConnOpen.Open();
            }
        }
        //Code for Connection Close.
        public void CloseCon(OleDbConnection ConnClose)
        {
            if (ConnClose.State == ConnectionState.Open)
            {
                ConnClose.Close();
                ConnClose.Dispose();
            }
        }
        public DataTable ExeQuery(string sqlQuery, string tableName)
        {
            try
            {
                connClass = new OleDbConnection(strDecryptedString);
                OleDbDataAdapter myAdapter = new OleDbDataAdapter(sqlQuery, connClass);
                DataSet dsData = new DataSet();
                myAdapter.SelectCommand.CommandTimeout = 0;
                myAdapter.Fill(dsData, tableName);
                dsData.Tables[0].TableName = tableName;
                myAdapter.Dispose();
                return dsData.Tables[0];
            }
            catch (Exception ex)
            {
                //                WriteToErrLog(ex);
                WriteErrorToEventLog("ExeQuery", ex.ToString());
                throw ex;
            }
            finally
            {
                CloseCon(connClass);
            }
        }
        // This Method is used to insert records by passing INSERT OleDb statement
        public void Execute_Non_Query(string sqlQuery)
        {
            connClass = new OleDbConnection(strDecryptedString);
            OleDbCommand ObjCommand;
            ObjCommand = connClass.CreateCommand();
            ObjCommand.Connection = connClass;
            ObjCommand.CommandText = sqlQuery;
            ObjCommand.CommandTimeout = 0;
            OpenCon(connClass);
            try
            {
                ObjCommand.ExecuteNonQuery();
            }
            catch (Exception ex)
            {
                WriteErrorToEventLog("Execute_Non_Query", ex.ToString());
                throw ex;
            }
            finally
            {
                CloseCon(connClass);
                ObjCommand.Dispose();
            }
        }
        public void SP_Execute(OleDbParameter[] paramCol, string strCmd)
        {
            connClass = new OleDbConnection(strDecryptedString);
            OleDbCommand cmd;
            cmd = connClass.CreateCommand();
            cmd.Connection = connClass;
            cmd.CommandType = CommandType.StoredProcedure;
            cmd.CommandText = strCmd;
            cmd.Parameters.AddRange(paramCol);
            cmd.CommandTimeout = 0;
            OpenCon(connClass);
            try
            {
                cmd.ExecuteNonQuery();
            }
            catch (Exception ex)
            {
                WriteErrorToEventLog("SP_Execute", ex.ToString());
                throw ex;
            }
            finally
            {
                CloseCon(connClass);
                cmd.Parameters.Clear();
                cmd.Dispose();
            }
        }
        public void SP_Execute_NoParam(string strCmd)
        {
            connClass = new OleDbConnection(strDecryptedString);
            OleDbCommand cmd;
            cmd = connClass.CreateCommand();
            cmd.Connection = connClass;
            cmd.CommandType = CommandType.StoredProcedure;
            cmd.CommandText = strCmd;
            cmd.CommandTimeout = 0;
            OpenCon(connClass);
            try
            {
                cmd.ExecuteNonQuery();
            }
            catch (Exception ex)
            {
                WriteErrorToEventLog("SP_Execute_NoParam", ex.ToString());
                throw ex;
            }
            finally
            {
                CloseCon(connClass);
                cmd.Dispose();
            }
        }
        public int SP_ExecuteWithReturn(OleDbParameter[] paramCol, string strCmd)
        {
            connClass = new OleDbConnection(strDecryptedString);
            OleDbCommand cmd;
            cmd = connClass.CreateCommand();
            cmd.Connection = connClass;
            cmd.CommandType = CommandType.StoredProcedure;
            cmd.CommandText = strCmd;
            cmd.Parameters.AddRange(paramCol);
            cmd.CommandTimeout = 0;
            OpenCon(connClass);
            int HolidayId = 0;
            try
            {
                HolidayId = (int)cmd.ExecuteScalar();
            }
            catch (Exception ex)
            {
                WriteErrorToEventLog("SP_ExecuteWithReturn", ex.ToString());
                throw ex;
            }
            finally
            {
                CloseCon(connClass);
                cmd.Parameters.Clear();
                cmd.Dispose();
            }
            return HolidayId;
        }
        public void WriteToErrLog(Exception ex)
        {
            connClass = new OleDbConnection(strDecryptedString);
            // Write the exeception to the Error Log table
            string strName = "TimeCheckDataProcessService : " + ex.ToString();
            string strQuery = "Insert Into TA_Tbl_ErrLog (ErrMessage, ErrSource, ErrStackTrace, ErrException)Values('" + strName.ToString().Replace("'", "''") + "','" + ex.Source.ToString().Replace("'", "''") + "', '" + ex.StackTrace.ToString().Replace("'", "''") + "', '" + ex.GetType().ToString().Replace("'", "''") + "')";
            OleDbCommand sqlcomWriteErrLog;
            sqlcomWriteErrLog = connClass.CreateCommand();
            sqlcomWriteErrLog.Connection = connClass;
            sqlcomWriteErrLog.CommandText = strQuery;
            sqlcomWriteErrLog.CommandTimeout = 0;
            OpenCon(connClass);
            try
            {
                sqlcomWriteErrLog.ExecuteNonQuery();
            }
            catch (Exception exc)
            {
                throw exc;
            }
            finally
            {
                CloseCon(connClass);
                sqlcomWriteErrLog.Dispose();
            }
        }
        public static void WriteErrorToEventLog(string applicationName, string msg)
        {
            StackFrame CallStack = new StackFrame(1, true);
            int lineNumber = CallStack.GetFileLineNumber();
            string msg2 = CallStack.GetFileName() + "\r\n";           // This get calling module's file name and adds to event log.
            msg2 += "Line: " + lineNumber.ToString() + "\r\n";        // This get calling module's line number of call to this function.
            msg2 += msg;
            if (!EventLog.SourceExists("TC2.7.0DataProcessLogSource"))
            {
                EventLog.CreateEventSource("TC2.7.0DataProcessLogSource", "TC2.7.0DataProcessLog");
            }
            EventLog.WriteEntry("TC2.7.0DataProcessLogSource", msg2, EventLogEntryType.Error, lineNumber);
        }
        public DataTable SelectCompanyDetails(string strCmd)
        {
            connClass = new OleDbConnection(strDecryptedString);
            OleDbCommand cmd;
            OleDbDataAdapter sqlAda = new OleDbDataAdapter();
            DataSet ds = new DataSet();
            cmd = connClass.CreateCommand();
            cmd.Connection = connClass;
            cmd.CommandType = CommandType.StoredProcedure;
            cmd.CommandText = strCmd;
            cmd.CommandTimeout = 0;
            OpenCon(connClass);
            try
            {
                sqlAda.SelectCommand = cmd;
                sqlAda.Fill(ds);
                sqlAda.SelectCommand.Connection.Close();
                sqlAda.SelectCommand.Dispose();
                sqlAda.Dispose();
                return ds.Tables[0];
            }
            catch (Exception ex)
            {
                WriteToErrLog(ex);
                throw ex;
            }
            finally
            {
                CloseCon(connClass);
                cmd.Parameters.Clear();
                cmd.Dispose();
            }
        }
        public DataTable SelectDatas(OleDbParameter[] paramCol, string strCmd)
        {
            connClass = new OleDbConnection(strDecryptedString);
            OleDbCommand cmd;
            OleDbDataAdapter sqlAda = new OleDbDataAdapter();
            DataSet ds = new DataSet();
            cmd = connClass.CreateCommand();
            cmd.Connection = connClass;
            cmd.CommandType = CommandType.StoredProcedure;
            cmd.CommandText = strCmd;
            cmd.Parameters.AddRange(paramCol);
            cmd.CommandTimeout = 0;
            OpenCon(connClass);
            try
            {
                sqlAda.SelectCommand = cmd;
                sqlAda.Fill(ds);
                sqlAda.SelectCommand.Connection.Close();
                sqlAda.SelectCommand.Dispose();
                sqlAda.Dispose();
                return ds.Tables[0];
            }
            catch (Exception ex)
            {
                WriteToErrLog(ex);
                throw ex;
            }
            finally
            {
                CloseCon(connClass);
                cmd.Parameters.Clear();
                cmd.Dispose();
            }
        }
        public int SP_ExecuteScalar(OleDbParameter[] paramCol, string strCmd)
        {
            int shiftId = 0;
            connClass = new OleDbConnection(strDecryptedString);
            OleDbCommand cmd;
            cmd = connClass.CreateCommand();
            cmd.Connection = connClass;
            cmd.CommandType = CommandType.StoredProcedure;
            cmd.CommandText = strCmd;
            cmd.Parameters.AddRange(paramCol);
            cmd.CommandTimeout = 0;
            OpenCon(connClass);
            try
            {
                shiftId = Convert.ToInt32(cmd.ExecuteScalar());
            }
            catch (Exception ex)
            {
                WriteToErrLog(ex);
                throw ex;
            }
            finally
            {
                CloseCon(connClass);
                cmd.Parameters.Clear();
                cmd.Dispose();
            }
            return shiftId;
        }
    }
}
