using System;
using System.Collections;
using System.Data;
using System.Data.Common;
using System.Data.SqlClient;
using System.Diagnostics;
using DataAccessLayer;
using Microsoft.Practices.EnterpriseLibrary.Data;
/// <summary>
/// Summary description for ExportData
/// </summary>
public class ExportData
{
    DataAccess objDataAccess = new DataAccess();
    Hashtable htable = new Hashtable();
    Database DBCon = DatabaseFactory.CreateDatabase("DbConnection");
    string strDecryptedString = string.Empty;
    public ExportData()
    {
        strDecryptedString = Cryptographys.Cryptography.GetDecryptedConnectionString();
        if (strDecryptedString != string.Empty)
        {
            ResetDatabaseConnection(strDecryptedString);
        }
    }
    public void ResetDatabaseConnection(string strDecryptedString)
    {
        DbProviderFactory objData = DBCon.DbProviderFactory;
        DBCon = new Microsoft.Practices.EnterpriseLibrary.Data.Sql.SqlDatabase(strDecryptedString);
    }
    public void Insert_CustomFieldData(DataTable dtCustomFieldData)
    {
        try
        {
            SqlBulkCopy CustomFieldData = new SqlBulkCopy(strDecryptedString, SqlBulkCopyOptions.TableLock);
            CustomFieldData.DestinationTableName = "TA_ExportData_FieldData";
            CustomFieldData.ColumnMappings.Clear();
            foreach (DataColumn dcColumn in dtCustomFieldData.Columns)
            {
                CustomFieldData.ColumnMappings.Add(new SqlBulkCopyColumnMapping(dcColumn.ColumnName, dcColumn.ColumnName));
            }
            Stopwatch sw = new Stopwatch();
            sw.Start();
            CustomFieldData.WriteToServer(dtCustomFieldData);
            sw.Stop();
        }
        catch (Exception Ex)
        {
            throw Ex;
        }
    }
    /// <summary>
    /// Delete the customer data based on the option type.
    /// </summary>
    /// <param name="OptionType"></param>
    public void DeleteCustomDataBasedOnOptionType(string OptionType, string DataType, int CompanyID)
    {
        try
        {
            htable.Clear();
            htable.Add("@OptionType", OptionType);
            htable.Add("@DataType", DataType);
            htable.Add("@CompanyID", CompanyID);
            objDataAccess.GetStoredProcCommand(htable, "DeleteCustomDataBasedOnOptionType");
        }
        catch (Exception ex)
        {
            throw ex;
        }
    }
    /// <summary>
    /// Inserts or updates the Custom format based on the option type.
    /// </summary>
    public void InsertUpdateCustomFormatForExportData(string OptionType, string DateFormat, string TimeFormat, string DateTimeFormat, string XMLRootTag, string FirstChildTag, string DataType, int CompanyID)
    {
        try
        {
            htable.Clear();
            htable.Add("@OptionType", OptionType);
            htable.Add("@DateFomat", DateFormat);
            htable.Add("@TimeFormat", TimeFormat);
            htable.Add("@DateTimeFormat", DateTimeFormat);
            htable.Add("@XMLRootTag", XMLRootTag);
            htable.Add("@XMLFirstChildTag", FirstChildTag);
            htable.Add("@DataType", DataType);
            htable.Add("@CompanyID", CompanyID);
            objDataAccess.GetStoredProcCommand(htable, "Insert_Update_CustomFormatForExportData");
        }
        catch (Exception ex)
        {
            throw ex;
        }
    }
    /// <summary>
    /// Fetches the custom format for export data based on the option type
    /// </summary>
    /// <param name="OptionType"></param>
    public DataSet GetCustomFormatsForExportData(string OptionType, string Datatype, int CompanyID)
    {
        try
        {
            DataSet dsCustomFormat = new DataSet();
            htable.Clear();
            htable.Add("@OptionType", OptionType);
            htable.Add("@DataType", Datatype);
            htable.Add("@CompanyID", CompanyID);
            dsCustomFormat = objDataAccess.GetStoredProcCommand("GetCustomFormatForExportData", htable);
            return dsCustomFormat;
        }
        catch (Exception ex)
        {
            throw ex;
        }
    }
    /// <summary>
    /// Fetches the custom fields for export data based on the option type
    /// </summary>
    /// <param name="OptionType"></param>
    public DataSet GetCustomFieldNamesForExportData(string OptionType, string Datatype, int CompanyID)
    {
        try
        {
            DataSet dsCustomField = new DataSet();
            htable.Clear();
            htable.Add("@OptionType", OptionType);
            htable.Add("@DataType", Datatype);
            htable.Add("@CompanyID", CompanyID);
            dsCustomField = objDataAccess.GetStoredProcCommand("GetCustomFieldNameForExportData", htable);
            return dsCustomField;
        }
        catch (Exception ex)
        {
            throw ex;
        }
    }
    /// <summary>
    /// Inserts or updates the FTP configuration.
    /// </summary>
    public void InsertUpdateFTPConfiguration(string Address, string UserName, string Password, string Port)
    {
        try
        {
            htable.Clear();
            htable.Add("@Address", Address);
            htable.Add("@UserName", UserName);
            htable.Add("@Password", Password);
            htable.Add("@Port", Port);
            objDataAccess.GetStoredProcCommand(htable, "Proc_TA_InsertUpdateFTPConfiguration");
        }
        catch (Exception ex)
        {
            throw ex;
        }
    }
    /// <summary>
    /// Select the FTP configuration
    /// </summary>
    /// <param name="OptionType"></param>
    public DataSet SelectFTPConfiguration()
    {
        try
        {
            DataSet dsFTPConfiguration = new DataSet();
            dsFTPConfiguration = objDataAccess.ExecuteDataSet("Proc_TA_SelectFTPConfiguration");
            return dsFTPConfiguration;
        }
        catch (Exception ex)
        {
            throw ex;
        }
    }
}